/*
 * Copyright 1996 John D. Polstra.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by John Polstra.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *      $Id: elf_386.h,v 1.2 1996/01/19 18:36:12 jdp Exp $
 */

#ifndef ELF_386_H /* { */
#define ELF_386_H 1

/*
 * ELF definitions specific to the i386 architecture.
 */

typedef struct {	/* Auxiliary vector entry on initial stack */
    int a_type;			/* AT_xxx, see below */
    union {
	long a_val;		/* AT_{EXECFD, PHENT, PHNUM, PAGESZ, FLAGS} */
	void *a_ptr;		/* AT_{PHDR, BASE, ENTRY} */
	void (*a_fcn)();	/* Never used */
    } a_un;
} auxv_t;

enum {			/* Values for a_type in auxiliary vector entry */
    AT_NULL,			/* Terminates the vector */
    AT_IGNORE,			/* Ignored */
    AT_EXECFD,			/* File descriptor of program to load */
    AT_PHDR,			/* Program header of program already loaded */
    AT_PHENT,			/* Size of each program header entry */
    AT_PHNUM,			/* Number of program header entries */
    AT_PAGESZ,			/* Page size in bytes */
    AT_BASE,			/* Interpreter's base address */
    AT_FLAGS,			/* Flags (unused for i386) */
    AT_ENTRY,			/* Where interpreter should transfer control */

    /*
     * The following non-standard values are used for passing information
     * from John Polstra's testbed program to the dynamic linker.
     */
    AT_BRK,			/* Starting point for sbrk and brk */
    AT_DEBUG,			/* Debugging level */

    AT_COUNT
};

/*
 * Relocation types.
 */

enum {
    R_386_NONE,
    R_386_32,
    R_386_PC32,
    R_386_GOT32,
    R_386_PLT32,
    R_386_COPY,
    R_386_GLOB_DAT,
    R_386_JMP_SLOT,
    R_386_RELATIVE,
    R_386_GOTOFF,
    R_386_GOTPC,

    R_386_COUNT
};

#endif /* } */
